Class: Safer
- Inherits:
-
Object
- Object
- Safer
- Defined in:
- lib/safer.rb,
lib/safer/ivar.rb
Overview
Safer is an umbrella library, whose components are designed to improve the safety of your code. There are at present three modules under the safer umbrella:
[Safer::IVar] generates specially-named accessor functions for class instance variables. [Safer::Protocol] is used to provide a ruby analogue to Objective-C Protocols (which are similar to Java interfaces, but do not require inheritance). [Safer::HashProtocol] verifies that a Hash keys follow a defined format. Intended to help use of Hash objects as keyword parameters.
Defined Under Namespace
Classes: HashProtocol, IVarFactory, Protocol
Constant Summary collapse
- VERSION =
Current release of Safer.
"0.4.1"- IVar =
Default IVarFactory object.
IVarFactory.new(IVarFactory::Prefix::Full)